/* ************************************************************************** */
// Programmer: Tobin Schwaiger-Hastanan (tobin@13hex.com)
// 
// Copyright (C) 2003 13Hex, Inc.
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions 
// are met:
// 
//      * Redistributions of source code must retain the above copyright 
//        notice, this list of conditions and the following disclaimer.
// 
//      * Redistributions in binary form must reproduce the above 
//        copyright notice, this list of conditions and the following 
//        disclaimer in the documentation and/or other materials 
//        provided with the distribution. 
// 
//      * Neither the name of 13Hex, Inc. nor the names 
//        of its contributors may be used to endorse or promote 
//        products derived from this software without specific prior 
//        written permission. 
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
// FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
// COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
// INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
// BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
// IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
// POSSIBILITY OF SUCH DAMAGE.
/* ************************************************************************** */
#ifndef _ENTITYFACTORY_DOT_H_INCLUDED_
#define _ENTITYFACTORY_DOT_H_INCLUDED_

namespace snbfrx
{
    /**
      * Abstract class EntityFactory that is used to create Entity objects.
      *
      * For more effecient handling of entities, I recommend that a pool of entities
      * are allocated upon factory initialization.  When the creation of an entity
      * is requested, the createEntity method should search the pool to see if a
      * free entity is available and then return that entity.
      *
      * Likewise when an entity is released, the factory should reset the data
      * for the entity and return it to the pool.
      */
    class EntityFactory
    {
    public:
        /**
          * Virtual constructor.
          */
        virtual ~EntityFactory();

        /**
          * returns the name of the entity factory.
          * The name is primarily used to manage factories in the EntityManager
          * class.
          *
          * @return constant reference to an stl string containing the factory name.
          */
        const std::string& getName() const { return m_name; }        

        /**
          * returns the description of the entity factory.
          *
          * @return constant reference to an stl string containing the description.
          */
        const std::string& getDescription() const { return m_description; }

        /**
          * Pure virtual function that creates an entity from a specific factory.
          *
          * @return pointer to a created entity object.
          */
        virtual Entity* createEntity() = 0;
        
        /**
          * Pure virtual function that releases an entity that this factory had
          * created.
          *
          * @param pentity pointer to an entity object that is to be released.          
          */
        virtual void releaseEntity( Entity* pentity ) = 0;

    protected:
        /**
          * Protected method for setting the name of the factory.
          *
          * @param name constant reference to an stl string containing the factory name.
          */
        void setName( std::string& name )  { m_name = name; }

        /**
          * Protected method for setting the description of the factory.
          *
          * @param description constant reference to an stl string containing the 
          * factory description.
          */
        void setDescription( std::string& description )  { m_description = description; }

    private:
        /** stl string containing the name of the factory */
        std::string m_name;
        
        /** stl string containing the description of the factory */
        std::string m_description;
    };
}

#endif